	LIST	n=85
;**********************************************************************
;
;	File:	FORDSONY.ASM
;
;	Desc:	Code for PIC12C508-based Ford-to-Sony stalk adapter.
;
;	Notes:	Features:
;		-	reads Ford stalk using RC circuit
;		-	re-reads until same input detected 64 times
;		-	controls Sony unit using GPIO outputs
;		-	sleeps for ~36 ms between reads if no buttons pressed
;
;**********************************************************************

	INCLUDE	p12c508.inc
	INCLUDE	fordsony.inc

	ORG	0
	movwf	OSCCAL			; calibrate internal oscillator
	goto	main

	PAGE
;**********************************************************************
;
;	Name:	test_mode
;
;	Desc:	RC circuit test mode after power-on-reset. Shows the
;		time constant by repeatedly charging and discharging the
;		capacitor.
;
;	Notes:	Test mode is exited by WDT timeout.
;
;**********************************************************************

	ORG	07F
test_mode	
	movlw	TRIS_CHARGE
	tris	GPIO			; start charging C
	nop				; 4 Tcy for charging
	nop
	movlw	TRIS_IDLE
	tris	GPIO			; start discharging C
test_loop
	btfsc	GPIO,RCINPUT		; wait until discharged
	goto	test_loop
	goto	test_mode		; infinite loop until WDT

	PAGE
;**********************************************************************
;
;	Name:	main
;
;	Desc:	Main program.
;
;	Notes:	-
;
;**********************************************************************

main	ORG	0FF

; initialize registers
	movlw	GPIO_VALUE
	movwf	GPIO
	movlw	OPTION_VALUE
	option

	btfsc	STATUS,NOT_TO		; power-on reset?
	goto	test_mode

; initialize variables
	movlw	TRIS_IDLE
	movwf	TRIS_IMAGE
;	tris	GPIO			; not required, ok already after reset
	movwf	LAST_INPUT
	clrf	INPUT_COUNTER

main_loop
	clrwdt
	movlw	TRIS_CHARGE
	andwf	TRIS_IMAGE,W
	tris	GPIO			; start charging C
	nop				; 4 Tcy for charging
	nop	
	movf	TRIS_IMAGE,W
	tris	GPIO			; start discharging C
	
	movlw	H'FF'			; space for tuning the loop count
	movlw	H'FF'
	movlw	VOL_DOWN_LOOP
	movwf	COUNTER
	movlw	TRIS_VOL_DOWN
loop1	decfsz	COUNTER,F
	goto	loop1
	btfss	GPIO,RCINPUT		; discharged?
	goto	done

	movlw	H'FF'			; space for tuning the loop count
	movlw	H'FF'
	movlw	VOL_UP_LOOP
	movwf	COUNTER
	movlw	TRIS_VOL_UP
loop2	decfsz	COUNTER,F
	goto	loop2
	btfss	GPIO,RCINPUT		; discharged?
	goto	done

	movlw	H'FF'			; space for tuning the loop count
	movlw	H'FF'
	movlw	SEEK_UP_LOOP
	movwf	COUNTER
	movlw	TRIS_SEEK_UP
loop3	decfsz	COUNTER,F
	goto	loop3
	btfss	GPIO,RCINPUT		; discharged?
	goto	done

	movlw	H'FF'			; space for tuning the loop count
	movlw	H'FF'
	movlw	SEEK_DOWN_LOOP
	movwf	COUNTER
	movlw	TRIS_SEEK_DOWN
loop4	decfsz	COUNTER,F
	goto	loop4
	btfss	GPIO,RCINPUT		; discharged?
	goto	done

	movlw	H'FF'			; space for tuning the loop count
	movlw	H'FF'
	movlw	MODE_LOOP
	movwf	COUNTER
	movlw	TRIS_SOURCE
loop5	decfsz	COUNTER,F
	goto	loop5
	btfsc	GPIO,RCINPUT		; discharged?

; not discharged, no button pressed
	movlw	TRIS_IDLE

done	xorwf	LAST_INPUT,F		; same as previous?
	movwf	LAST_INPUT
	btfss	STATUS,Z
	goto	not_same
	movf	INPUT_COUNTER,F		; enough same readings?
	btfss	STATUS,Z
	goto	count

; enough same readings, set output pins
	tris	GPIO
	movwf	TRIS_IMAGE
	xorlw	TRIS_IDLE		; any button pressed
	btfsc	STATUS,Z
	sleep				; no, sleep until WDT
	goto	main_loop		; yes, re-read input immediately

; different input than previous
not_same
	movlw	REPEAT_COUNT
	movwf	INPUT_COUNTER
; not enough same inputs yet, count down
count	decf	INPUT_COUNTER,F
	goto	main_loop

;    ---***---
	END

